
function trim (string) {
  return string.substring(1, string.length - 1)
}

function style (rx, tag) {
  return function (text) {
    var m = rx.exec(text)
    while (m) {
      text = text.replace(m[0], '<'+tag+'>'+trim(m[0])+'</'+tag+'>')
      m = rx.exec(text)
    }
    return text
  }
}

var italics = style(/_[^_]+_/, 'em')
var bold    = style(/\*[^\*]+\*/, 'strong')
var code    = style(/`[^`]+`/, 'code')

function prefix (rx, tag) {
  return function (val) {
    var m = rx.exec(val)
    if(m) {
      var l = m[0].length
      //oh yeah, escape chars...
      var t = 'function' === typeof tag ? tag(m) : tag
      return '<'+t+'>'+val.substring(l)+'</'+t+'>'
    }
    return val
  }
}

var header = prefix(/^#+/, function (m) { return 'H'+m[0].length })
var li = prefix(/^\s*-+/,  'li')

function styles (text) {
  return header(li(bold(italics(code(text)))))
}

function toHTML (val) {
  if('\n' === val)
    return '<br/>'
  return styles(val)
}

function defaultTemplate (val, key, el) {
  if(el)
    return el.innerHTML = toHTML(val), el
  var el = document.createElement('span')
  el.innerHTML = toHTML(val)
  return el
}

var widget = require('r-array/widget')

module.exports = function (rEdit, template) {
  return widget(rEdit, template || defaultTemplate)
}
